from hamcrest import assert_that, is_

from nolog.logic_engine.conjunction import And
from nolog.logic_engine.disjunction import Or
from nolog.logic_engine.negation import Not
from nolog.tests.conftest import Truthy, Falsy


def test_simple_disjunction():
    prop = Or(Truthy(), Truthy())

    assert_that(prop.eval(), is_(True))


def test_partial_disjunction():
    prop = Or(Truthy(), Falsy())

    assert_that(prop.eval(), is_(True))


def test_disjunction_with_negative():
    prop = Or(Falsy(), Falsy())

    assert_that(prop.eval(), is_(False))


def test_disjunction_with_negation():
    prop = Or(Falsy(), Not(Falsy()))
    assert_that(prop.eval(), is_(True))


def test_disjunction_with_negation_unordered():
    prop = Or(Not(Falsy()), Truthy())
    assert_that(prop.eval(), is_(True))


def test_disjunction_of_disjunctions():
    prop = Or(Truthy(), Or(Truthy(), Falsy()))
    assert_that(prop.eval(), is_(True))


def test_disjunction_of_disjunctions_with_negation():
    prop = Or(Truthy(), Or(Not(Truthy()), Falsy()))
    assert_that(prop.eval(), is_(True))


def test_disjunction_of_disjunctions_with_more_predicates():
    prop = Or(Not(Falsy()), Truthy(), Or(Truthy(), Falsy()))
    assert_that(prop.eval(), is_(True))


def test_disjunction_of_disjunctions_with_more_predicates_and_negation():
    prop = Or(Not(Falsy()), Truthy(), Or(Not(Truthy()), Falsy()))
    assert_that(prop.eval(), is_(True))


def test_disjunction_of_conjunctions():
    prop = Or(And(Truthy(), Not(Falsy())), Falsy())
    assert_that(prop.eval(), is_(True))


def test_disjunction_of_failing_conjunctions():
    prop = Or(And(Truthy(), Falsy()), Falsy())
    assert_that(prop.eval(), is_(False))


def test_disjunction_of_conjunctions_with_truthy_value():
    prop = Or(And(Truthy(), Falsy()), Truthy())
    assert_that(prop.eval(), is_(True))



from typing import Any, Optional

from nolog.logic_engine.predicate import Predicate


class Or(Predicate):
    def __init__(self, *args):
        predicates = args
        self.predicates = predicates

    def eval(self, variable: Optional[Any] = None) -> bool:
        return any(predicate.eval(variable) for predicate in self.predicates)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['nolog', 'nolog.logic_engine', 'nolog.tests']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'nolog',
    'version': '0.1.3',
    'description': 'A native Logic engine for Python',
    'long_description': '# NoLog\n\nA propositional logic implementation in Python, to be used as a rule engine.',
    'author': 'Dragos Dumitrache',
    'author_email': 'dragos@afterburner.dev',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



import abc
import dataclasses
from typing import Any, List, Optional


@dataclasses.dataclass
class Predicate(abc.ABC):
    accepted_values: List[Any] = dataclasses.field(default_factory=list)

    @abc.abstractmethod
    def eval(self, variable: Optional[Any] = None) -> bool:
        raise NotImplementedError
